import sympy as sp

def E(x, f, l, r):
    return sp.integrate(x*f, ('x', l, r))

def D(x, f, l, r):
    # return E((x - E(x, f, l, r))**2, f, l, r)
    return E(x**2, f, l, r) - E(x, f, l, r)**2

def P(f, l, r):
    return lambda a, b: float(sp.integrate(f, ('x', max(l, a), min(r, b))))

def cdist(f, l, r):
    f = sp.sympify(f)
    с, = sp.solve(sp.integrate(f, ('x', l, r))-1)
    f = f.subs('с', с)
    x, = f.free_symbols
    return {'c': с, 'E': E(x, f, l, r), 'D': D(x, f, l, r)}, P(f, l, r)



class Probability:

    def __init__(self, p):
        self.p = p

    def __and__(self, other):
        return Probability(self.p * other.p)

    def __or__(self, other):
        return Probability(self.p + other.p - self.p * other.p)

    def __invert__(self):
        return Probability(1 - self.p)

import re
def logic(expr, values, factor=1):
    expr = re.sub('(\\d)+', 'P[\\1]', expr)
    P = [Probability(v*factor) for v in values]
    return f'{eval(expr).p:.3f}'



from fractions import Fraction
from . tools import B, P, L, Pe, Le

def BPL(*args, **kwargs):
    assert len(args) >= 2 and isinstance(args[1], str) or \
        'p' in kwargs and isinstance(kwargs['p'], str)
    b = B(*args, **kwargs)
    p = P(*args, **kwargs), Pe(*args, **kwargs)
    l = L(*args, **kwargs), Le(*args, **kwargs)
    r = f'B: {b:.3f}, ' \
        f'P: {p[0]:.3f}, Pl: {max(p[0]-p[1], 0):.3f}, Pr: {min(p[0]+p[1], 1):.3f}, ' \
        f'L: {l[0]:.3f}, Ll: {max(l[0]-l[1], 0):.3f}, Lr: {min(l[0]+l[1], 1):.3f}'
    return r



from fractions import Fraction
import math

def C(k, n):
    assert k <= n
    return math.factorial(n) // math.factorial(k) // math.factorial(n-k)

def B(n, p, f, t=0):
    p = Fraction(p)
    t = t or f
    q = 1 - p
    return float(sum(C(i, n) * p**i * q**(n-i) for i in range(f, t+1)))

def P(n, p, f, t=0):
    p = Fraction(p)
    t = t or f
    l = n * p
    s = sum(l**i / math.factorial(i) for i in range(f, t+1))
    return float(s / Fraction(math.e)**l)

def L(n, p, f, t=0, d=1000):
    p = float(p)
    t = t or f
    t -= d > 1
    q = 1 - p
    result = 0
    np = n*p
    sqrtnpq = math.sqrt(n*p*q)
    for i in range(f, t+1):
        for j in range(d):
            x = (i+j/d - np) / sqrtnpq
            result += math.exp(-(x**2) / 2) / d
    return result / math.sqrt(n*p*q*2*math.pi)

def Pe(n, p, *args, **kwargs):
    p = float(p)
    return min(p, n*p**2)

def Le(n, p, *args, **kwargs):
    p = float(p)
    q = 1 - p
    return 1 / (p*q*math.sqrt(n))



from fractions import Fraction
import numpy as np

def get_row(row):
    return tuple(Fraction(col) for col in row.split(','))

def get_table(table):
    return tuple(get_row(row) for row in table.split(';'))

def calc(array, values, axis):
    P = np.sum(array, axis=axis)
    E = np.sum(values * P)
    D = np.sum(P * (values-E)**2)
    return P, E, D

def table(e, n, p):
    p = p.replace('x', '-1')
    e = np.array(get_row(e))
    n = np.array(get_row(n))
    p = np.array(get_table(p))
    x = -np.sum(p)
    p[p == -1] += x + 1
    Pe, Ee, De = calc(p, e, 0)
    Pn, En, Dn = calc(p, n, 1)
    cov = np.sum(p*e*n.reshape(-1, 1)) - Ee*En
    rho = cov / (De*Dn) ** 0.5
    return f'x: {x}, E: {Ee}, D: {De}, E: {En}, D: {Dn}, cov: {cov}, rho: {rho:.3f}'



from matplotlib import pyplot as plt
import numpy as np
from scipy import stats
import requests


def get(url):
    raw = requests.get(url).content.decode()
    return np.array(raw.strip().split('\n'), float)

def task1(dist):
    n = len(dist)
    E = np.sum(dist) / n
    D = np.sum((dist-E)**2) / (n-1)
    return {'E': E, 'D': D}

def task2(dist, ncol=10):
    plt.figure(label='Task 2')
    plt.bar_label(plt.hist(dist, ncol)[2])
    plt.show(block=False)

def task3(dtype, E, D):
    if dtype == 'N':
        return E, D
    if dtype == 'E':
        return 1/E, -999
    if dtype == 'U':
        u = (3*D)**.5
        return E-u, E+u

def task4(dist, dtype, E, D, eps=0.05):
    n = len(dist)
    if dtype == 'N':
        return E-stats.t.ppf(1-(eps/2), n-1)*(D/n)**.5, D/stats.chi2.ppf(eps/2, n-1)*(n-1)
    if dtype == 'E':
        return 1/E-stats.norm.ppf(1-eps/2)/n**.5/E, -999
    if dtype == 'U':
        u = stats.norm.ppf(1-eps/2)*(max(dist)-min(dist))/(3*n)**.5
        return 2*E-u-max(dist), 2*E+u-min(dist)

def parse(url):
    dist = get(url)
    args = task1(dist)
    task2(dist)
    dtype = input('dtype = ').upper()
    print('Task 1:', tuple(args.values()))
    print('Task 3:', task3(dtype, **args))
    print('Task 4:', task4(dist, dtype, **args))
